/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 11-Sep-2004
 *
 */
package com.exponto.jstock.db;

/**
 * @author kaare
 *
 *Used to load parametervalues to/from database. In order to keep the db 
 *related code away from the indicatorparameter classes, this class will be 
 *used for that.
 *
 *It is important that if the parameters of an indicator is changed, the 
 *hashcode must be updated, otherwise e.g. lines would be completly 
 *wrong.
 *
 *A negative index means that this parameter is new to the database.
 */
public class DBIndicatorParameters {
	private int m_index = -1; //The database index, -1 for new parameters
	private boolean m_isDeleted = false; // set to true when the parameter shall be deleted
	private String m_IndicatorName; // The name of the indicators the parameters belongs to
	private String m_parameterValue; // A String with all parametersvalue, it's up to the IndicatorParameter class of the Indicator to decode the string.
	private int m_indicatorCode;	// Hashcode of parameters used to keep different panels of same indicator apart
	private boolean m_isDisplayed = true; // True if this indicator shall be displayed
	
	public DBIndicatorParameters() {
		
	}
	/**
	 * Creates a new DBIndicatorParameters. 
	 * 
	 * @param index - The db index
	 * @param indicator - name of the indicator
	 * @param value -	Astring representing the parameters of the indicator.
	 * @param code - hash code of the indicator parameters
	 * @param isDisplayed - true if this indicator shall be displayed, false otherwise
	 * The indicators shall be able to create and decode this string.
 */
	public DBIndicatorParameters(int index, String indicator, String value, int code, boolean isDisplayed) {
		m_index = index;
		m_IndicatorName = indicator;
		m_parameterValue =value;
		m_indicatorCode = code;
		m_isDisplayed = isDisplayed;
	}
	
	public boolean equals(Object o) {
		boolean result = true;
		DBIndicatorParameters value = (DBIndicatorParameters) o;
		if (this.m_index != value.getIndex()) {
			result =false;
		}
		if (!this.m_IndicatorName.equals(value.getIndicatorName())) {
			result =false;
		}
		if (!this.m_parameterValue.equals(value.getParameterValue())) {
			result =false;
		}
		if (this.isDeleted() != value.isDeleted()) {
			result =false;
		}
		if (this.m_indicatorCode != value.getIndicatorCode()) {
			result =false;
		}
		return result;
	}
	
	
	public int hashCode() {
		int result = 775;
		if (isDeleted()) {
			result ++;
		}
		result = result + m_index + m_IndicatorName.hashCode() + m_parameterValue.hashCode() + m_indicatorCode;
		return result;
	}
	/**
	 * Compares this DBIndicatorParameters with anotherone and return true
	 * if parameter value and indicator are equal.
	 * 
	 * Note that this methode don't care about the db index etc.
	 * 
	 * @param dbIndParam the one to compare with
	 * @return true if this is the same parameter false otherwise
	 */
	public boolean compareIndicator(DBIndicatorParameters dbIndParam) {
		boolean result = true;
		if (!this.m_IndicatorName.equals(dbIndParam.getIndicatorName())) {
			result =false;
		} else if (!this.m_parameterValue.equals(dbIndParam.getParameterValue())) {
			result =false;
		}
		return result;
	}
	
	/**
	 * @return Returns the index.
	 */
	public int getIndex() {
		return m_index;
	}
	/**
	 * @param Index The index to set.
	 */
	public void setIndex(int index) {
		this.m_index = index;
	}
	/**
	 * @return Returns the indicatorName.
	 */
	public String getIndicatorName() {
		return m_IndicatorName;
	}
	/**
	 * @param name The IndicatorName to set.
	 */
	public void setIndicatorName(String name) {
		m_IndicatorName = name;
	}
	/**
	 * @return Returns the parameterValue.
	 */
	public String getParameterValue() {
		return m_parameterValue;
	}
	/**
	 * @param value The parameterValue to set.
	 */
	public void setParameterValue(String value) {
		m_parameterValue = value;
	}

	/**
	 * @return Returns the isDeleted.
	 */
	public boolean isDeleted() {
		return m_isDeleted;
	}
	/**
	 * 
	 */
	public void delete() {
		m_isDeleted = true;
	}
	
	public void undelete() {
		m_isDeleted = false;
	}
	/**
	 * @return Returns the m_indicatorcode.
	 */
	public int getIndicatorCode() {
		return m_indicatorCode;
	}
	/**
	 * @param m_indicatorcode The m_indicatorcode to set.
	 */
	public void setIndicatorCode(int indicatorcode) {
		this.m_indicatorCode = indicatorcode;
	}

	/**
	 * @param boolean
	 */
	public void setDisplayed(boolean b) {
		m_isDisplayed = b;
	}
	public boolean isDisplayed() {
		return m_isDisplayed;
	}
}
