/*
 *                                 jStock
 *                    (C) Copyright  2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 12.jul.2004
 *
 */
package com.exponto.jstock.gui.chartinfo;

import org.apache.log4j.Logger;

import com.exponto.jstock.indicators.IndicatorMaster;

/**
 * The Rule class contains data for one rule. Each rule consist of a name
 * and two logical subexpressions. The two subexpressions is always tied 
 * together with a logical AND, so both has to be true if the rule shall
 * evalute to true. 
 * 
 * The second subexpression may be empty and is then considered to be true. 
 * 
 * The class is responible to collect any data it need to evalute it self.
 * 
 * @author ttranu
 *
 */
public class Rule {
	private String ruleName;
	private String firstIndicator;
	private String firstOperator;
	private Double firstThreshold;
	private String secondIndicator;
	private String secondOperator;
	private Double secondThreshold;
	
	static Logger logger = Logger.getLogger(Rule.class);
	/**
	 * 
	 */
	public Rule() {
		super();
	}
	
	public Rule(String name) {
		ruleName = name;
	}
	public Rule(String name, String fInd, String fOp, Double fThr, String sInd, String sOp, Double sThr ) {
		ruleName = name;
		firstIndicator = fInd;
		firstOperator =fOp;
		firstThreshold = fThr;
		secondIndicator = sInd;
		secondOperator =sOp;
		secondThreshold = sThr;
	}
	
	/**
	 * Evaluates a rule with a given input parameter. Note that the first 
	 * subexpression is default set to false and the second is set to 
	 * true. The if all or some of the parameters of the second 
	 * subexpression are null, the subexpression isn't evaluated and the 
	 * default is used.
	 * 
	 * @return boolean true if the rule evalutes to true, false otherwise
	 */
	public boolean evalute(){
		boolean fRes = false;
		boolean sRes = true;
		if ((firstIndicator != null) && (firstOperator != null) && (firstThreshold != null)) {
			Double value = getIndicatorValue(firstOperator);
			fRes = this.evaluteSubexpression(firstIndicator, firstOperator, firstThreshold.doubleValue());
		}
		if ((secondIndicator != null) && (secondOperator != null) && (secondThreshold != null)) {
			//Double value = getIndicatorValue(secondIndicator);
			sRes = this.evaluteSubexpression(secondIndicator, secondOperator, secondThreshold.doubleValue());
		}		
		return fRes && sRes;
	}
	/**
	 * 
	 * @return Returns the complete rule as a String[]
	 */
	public String[] getRule () {
		String [] strA = new String[7];
		strA[0] = ruleName;
		strA[1] = firstIndicator;
		strA[2] = firstOperator;
		strA[3] = firstThreshold.toString();
		strA[4] = secondIndicator;
		strA[5] = secondOperator;
		strA[6] = secondThreshold.toString();
		return strA;
	}
	
	public static String[] getColumnNames() {
		return Constants.columnNames;
	}
	
	public static String[] getOperators() {
		return Constants.operators;
	}
	
	public static String[] getIndicators() {
// TODO: Clean up
		return IndicatorMaster.getIndicatorsList();
	}
	
	/**
	 * @return Returns the firstIndicator.
	 */
	public String getFirstIndicator() {
		return firstIndicator;
	}
	/**
	 * @param firstIndicator The firstIndicator to set.
	 */
	public void setFirstIndicator(String firstIndicator) {
		this.firstIndicator = firstIndicator;
	}
	/**
	 * @return Returns the firstOperator.
	 */
	public String getFirstOperator() {
		return firstOperator;
	}
	/**
	 * @param firstOperator The firstOperator to set.
	 */
	public void setFirstOperator(String firstOperator) {
		this.firstOperator = firstOperator;
	}
	/**
	 * @return Returns the firstThreshold.
	 */
	public Double getFirstThreshold() {
		return firstThreshold;
	}
	/**
	 * @param firstThreshold The firstThreshold to set.
	 */
	public void setFirstThreshold(Double firstThreshold) {
		this.firstThreshold = firstThreshold;
	}
	/**
	 * @return Returns the ruleName.
	 */
	public String getRuleName() {
		return ruleName;
	}
	/**
	 * @param ruleName The ruleName to set.
	 */
	public void setRuleName(String ruleName) {
		this.ruleName = ruleName;
	}
	/**
	 * @return Returns the secondIndicator.
	 */
	public String getSecondIndicator() {
		return secondIndicator;
	}
	/**
	 * @param secondIndicator The secondIndicator to set.
	 */
	public void setSecondIndicator(String secondIndicator) {
		this.secondIndicator = secondIndicator;
	}
	/**
	 * @return Returns the secondOperator.
	 */
	public String getSecondOperator() {
		return secondOperator;
	}
	/**
	 * @param secondOperator The secondOperator to set.
	 */
	public void setSecondOperator(String secondOperator) {
		this.secondOperator = secondOperator;
	}
	/**
	 * @return Returns the secondThreshold.
	 */
	public Double getSecondThreshold() {
		return secondThreshold;
	}
	/**
	 * @param secondThreshold The secondThreshold to set.
	 */
	public void setSecondThreshold(Double secondThreshold) {
		this.secondThreshold = secondThreshold;
	}
	/**
	 * Must be updated if new operators is implemented
	 * String [] operators = {"greater than", "less than", "equals", "not equals", "rising", "sinking"};
	 * @param operator
	 * @param value
	 * @param threshold
	 * @return
	 */
	private boolean evaluteSubexpression(String indicator, String operator, double threshold){
		boolean result = false;
		double value = getIndicatorValue(indicator).doubleValue();
		// Greater than
		if (operator.equals(Constants.operators[0])){
			if (value > threshold) {
				result = true;
			}
		}else if (operator.equals(Constants.operators[1])){
			// less than
			if (value < threshold) {
				result = true;
			}
		}else if (operator.equals(Constants.operators[2])){
			// equals 
			// Might be difficult as it is hard for to doubles to be equal
			if (value == threshold) {
				result = true;
			}
		}else if (operator.equals(Constants.operators[3])){
			// not equal
			if (value != threshold) {
				result = true;
			}
		}else if (operator.equals(Constants.operators[4])){
			// rising
			// TODO: Implement rising
			
		}else if (operator.equals(Constants.operators[5])){
			// sinking
			// TODO: Implement sinking			
		}else {
			logger.error("Unknown operator");
		}
		return result;
	}
	
	private Double getIndicatorValue(String indicator) {
		Double result = null;
		//IndicatorController indController = IndicatorController.getInstance();
		//indController.
		// TODO: Implement this. Need to get the indicators with populated 
		//dataseries. They are currently in the ChartPanel/IndicatorPanel
		//or CandlestickPanel class. Probably not the right place
		return result;
	}

}
