/*
 *                                 jStock
 *                    (C) Copyright 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * IndicatorPanel.java
 *
 */

package com.exponto.jstock.gui.panels;

/**
 * Abstract super class for indicator panels. 
 * 
 * All that is needed for the indicator panels is to override the paint methode.
 * 
 * Note that the superclass constructor has to be called explicit
 * 
 * @author  kaare
 * @version 
 */

import java.awt.image.BufferedImage;
import javax.swing.JPanel;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import com.exponto.jstock.data.ChartData;
import com.exponto.jstock.indicators.Indicator;
import com.exponto.jstock.indicators.IndicatorAxisInfo;
import com.exponto.jstock.indicators.IndicatorLinesInterface;
import com.exponto.jstock.indicators.IndicatorParameters;
import com.exponto.jstock.indicators.LogDataSeries;
import com.exponto.jstock.indicators.LogTradingData;
import com.exponto.jstock.io.UserPreferences;
import com.exponto.util.Date;

import org.apache.log4j.Logger;
 
public abstract class AbstractIndicatorPanel extends JPanel{
	protected Indicator m_indicator;
	protected GraphicsParameters m_graphicsParameters;
	protected GraphicsParameters m_oldGraphicsParameters;
	protected BufferedImage m_bImage = null;
	protected static Logger logger = Logger.getLogger(AbstractIndicatorPanel.class);
	protected static UserPreferences userPrefs = UserPreferences.getInstance();
	//LogDataSeries lDS;
	protected ChartData m_cD;
	protected int m_tMargin;
	protected int m_bMargin;
    
	protected int m_dailyPrintout = -1;
	// For handling of lines
	protected Lines m_drawnLines;
	protected double m_x1 = -1.0; // Negative value here to indicate no valid new line
	protected double m_x2 = 0.0;
	protected double m_y1 = 0.0;
	protected double m_y2 = 0.0;
	
    /** Creates new Panel */
	public AbstractIndicatorPanel(ChartData chartData, Indicator ind) {
    	m_cD = chartData;
    	m_indicator = ind;
		m_indicator.initiateIndicator(m_cD.getLogDataSeries());
		//repaint();
		int hash;
		IndicatorParameters iP = ind.getIndicatorParameters();
		if (iP == null) {
			hash = 0;
		} else {
			hash = iP.hashCode();
		}
			m_drawnLines = new Lines(m_cD.getTickerID() , m_cD.getPanelType(), ind.getIndicatorName(), hash);
		if (userPrefs.getEnableDB()) {
			//JstockDB jDB = JstockDB.getInstance();
			m_drawnLines.loadLines(); 
		}
		setVisible(true);
    }
    
	
    /**
     * 
     * 
     * @param gP GraphicsParameters
     */
    public void setGraphicsParameters (GraphicsParameters gP) {
		m_graphicsParameters = gP;
		this.setPreferredSize(gP.getDimension());
		this.setMaximumSize(gP.getDimension());
		m_graphicsParameters.setBottomMargin(m_bMargin);
		m_graphicsParameters.setTopMargin(m_tMargin);
		//logger.debug("GraphicsParameters (dim, top, bottom margin"+ gP.getDimension().toString() + ", " + graphicsParameters.getTopMargin() + ", " + graphicsParameters.getBottomMargin());
    }

	public void updateDailyPrintout(int number) {
		m_dailyPrintout = number;
		repaint();
		//this.markDailyPrintout()
	}
    
	public void paint(Graphics gr) {
		super.paint(gr); 
	}

	protected boolean imageValid(Indicator ind) {
		boolean isValid = true;
		// Initialise oldGrahicsParameters on first run (will always be equal)
		if (m_oldGraphicsParameters == null) {
			m_oldGraphicsParameters = (GraphicsParameters) m_graphicsParameters.clone();
			isValid = false;
			//logger.debug("Image is not valid due to: No oldGraphics" );
		} 
		if (m_bImage == null) {
			isValid = false;
			//logger.debug("Image is not valid due to: bImage is null" );
		}
		// Check if indicator parameters has changed
		if (ind.hasChanged()) {
			isValid = false;
			ind.initiateIndicator(m_cD.getLogDataSeries());
			if (m_drawnLines != null) {
				m_drawnLines.saveLines();
				IndicatorParameters indParam =  ind.getIndicatorParameters();
				if (indParam != null) {
					m_drawnLines.reloadLines(ind.getIndicatorName(), indParam.hashCode());
				}
			}
			//logger.debug("Image is not valid due to: Indicator has changed " + ind.getIndicatorName());
		}
		if (!m_graphicsParameters.isEqual(m_oldGraphicsParameters)) {
			isValid = false;
			//logger.debug("Image is not valid due to: Graphics has changed" );
		} 
		//logger.debug("Image is " + isValid);
		return isValid;
	}
	
	protected void markDailyPrintout(final Graphics2D g2d){
		if (m_dailyPrintout >= 0) {
			Line2D line = new Line2D.Double ();
			int x =m_graphicsParameters.getLeftMargin() + m_graphicsParameters.getPixelPrDay() * m_dailyPrintout -1;
			double y0 = m_graphicsParameters.getDimension().getHeight() - m_bMargin;// + tMargin;
			double y1 = m_tMargin;
			// orange may give better visibility for the candlestick itself
			g2d.setColor(Color.gray);
			double y;
			boolean draw = true;
			// Draw dashed line
			for (y = y1; y < y0; y = y + 4) {
				if (draw) {
					line.setLine(x, y, x, y+4);    
					g2d.draw(line);
					line.setLine(x + m_graphicsParameters.getPixelPrDay(), y, x + m_graphicsParameters.getPixelPrDay(), y + 4);
					g2d.draw(line);
				}
				draw = !draw;
			}
			// Use this for solid line
//			line.setLine(x, y1, x + graphicsParameters.getPixelPrDay(), y1);
//			g2d.draw(line);
			g2d.setColor(Color.black); 
		}
	}
	
	protected void drawLines(final Graphics2D g2d) {
		// See comment in Lines class (addLine)
		// Also need to be able to delete lines
		// Print lines if in draw mode and line is "complete"
		//TODO: Lines has to saved and reloaded if the indicatorparameters are changed
		if (userPrefs.getDrawMode()) {
			m_drawnLines.reset();
			PersistentLine pL = m_drawnLines.hasNext();
			//Line2D line = new Line2D.Double ();
			Line2D line;
			int selectedEnd;
			Rectangle2D rect = new Rectangle2D.Double();
//			System.out.println(" Retrieved lines: ");
			while (pL != null ) {
				line = this.convertPersistentLine(pL);    
//				pL.setSelected(true);
				if(!pL.isDeleted()) {
					if (pL.isSelected()) {
						selectedEnd = pL.getSelected();
						//Line2D border = new Line2D.Double();
						int delta = 3;
						if (selectedEnd == 2) {
//							Other end selected
							g2d.setColor(Color.DARK_GRAY);
						} else {
							g2d.setColor(Color.yellow);
						}
						double x = line.getX1();
						double y = line.getY1();
						rect.setRect(x-delta, y-delta, 2*delta, 2*delta);
						g2d.fill(rect);
						if (selectedEnd == 1) {
							// Other end selected
							g2d.setColor(Color.DARK_GRAY);
						} else {
							g2d.setColor(Color.yellow);
						}
						x = line.getX2();
						y = line.getY2();
						rect.setRect(x-delta, y-delta, 2*delta, 2*delta);
						g2d.fill(rect);
						g2d.setColor(Color.black);
					}
					g2d.setColor(Color.black);
					g2d.draw(line);
				}
				pL = m_drawnLines.hasNext();
//				System.out.println(" Line: " + line.getX1() + " " + line.getY1() + ", " + line.getX2() + " " + line.getY2());
			}
			// For temporary printing while "dragging" line
			if (m_x1 >= 0) {
				g2d.setColor(Color.black);
				Line2D tempLine = new Line2D.Double();
				tempLine.setLine(m_x1, m_y1, m_x2, m_y2);    
				g2d.draw(tempLine);
			}
		}
	}

	/**
	 * Converts the (pixel) start and end points of a line to a persistent line that
	 * can be stored a database and be redrawn later.
	 * 
	 * @param x1 A double representing the x1 coordinate
	 * @param y1 A double representing the y1 coordinate
	 * @param x2 A double representing the x2 coordinate
	 * @param y2 A double representing the y2 coordinate
	 * @return a PersistentLine representing the input parameters
	 */
	private PersistentLine convertPoints(double lx1, double ly1, double lx2, double ly2) {
		int pixPrDay = userPrefs.getPixelPrStick();
		int lMargin = userPrefs.getLMargin();
		IndicatorAxisInfo indicatorAxisInfo;
		try {
			IndicatorLinesInterface iLI = (IndicatorLinesInterface) m_indicator;
			indicatorAxisInfo = iLI.getIndicatorAxisInfo();
		} catch (Exception e) {
			logger.info("Seems that this indicator does not support the IndicatorLinesInterface");
			logger.info(e.toString());
			return null;
		}
		double height = this.getHeight()  - m_bMargin - m_tMargin; 
		Date dateX2;
		double valueY1 = 0.0;
		double valueY2 = 0.0;
		// Find daynumbers
		LogDataSeries logDS = m_cD.getLogDataSeries();
		int startOfSeries = logDS.size() - 1 + m_graphicsParameters.getRelativeStartOfPeriode();
		int dayNumber = (int) Math.round((lx1-lMargin)/pixPrDay) + startOfSeries;
		LogTradingData lTD = logDS.getResult(dayNumber);
		//Date tempDate = lTD.getDate();
		Date dateX1 = lTD.getDate().klone();
		dayNumber = (int) Math.round((lx2-lMargin)/pixPrDay) + startOfSeries;
		if (dayNumber >= logDS.getListSize()) {
			int field;
			if(logDS.isWeekly()) {
				field = java.util.Calendar.WEEK_OF_YEAR;
			} else {
				field = java.util.Calendar.DAY_OF_MONTH;
			}
			lTD = logDS.getResult(logDS.getListSize()-1);
			dateX2 = Date.getNewDate(lTD.getDate(), field, dayNumber - logDS.getListSize());
		
		} else {
			lTD = logDS.getResult(dayNumber);
			dateX2 = lTD.getDate().klone();
		}
		if (indicatorAxisInfo.getScale().equals("linear")) {
			double absoluteValue = indicatorAxisInfo.getMaxValue() - indicatorAxisInfo.getMinValue();
			valueY1 = (height + m_tMargin - ly1)/height * absoluteValue + indicatorAxisInfo.getMinValue();
			valueY2 = (height + m_tMargin - ly2)/height * absoluteValue + indicatorAxisInfo.getMinValue();
		} else if (indicatorAxisInfo.getScale().equals("log")){
			double logMin =m_graphicsParameters.getLogaritmicMinimumValue();
			double scaleY = height/(m_graphicsParameters.getLogaritmicMaximumValue() - logMin);
			valueY1 = Math.exp(((height + m_tMargin - ly1)/ scaleY + logMin) * Constants.log10);
			valueY2 = Math.exp(((height + m_tMargin - ly2)/ scaleY + logMin) * Constants.log10);
		}
		PersistentLine pL = new PersistentLine(dateX1, valueY1, dateX2, valueY2);
		//System.out.println("ConvertPoints x1: " + dateX1.toString() +" y1: " + valueY1 + " x2: " + dateX2.toString() + " y2: " + valueY2);
		return pL;
	}
	
	private Line2D convertPersistentLine(PersistentLine pL) {
		int pixPrDay = userPrefs.getPixelPrStick();
		int lMargin = userPrefs.getLMargin();
		IndicatorAxisInfo indicatorAxisInfo;
		try {
			IndicatorLinesInterface iLI = (IndicatorLinesInterface) m_indicator;
			indicatorAxisInfo = iLI.getIndicatorAxisInfo();
		} catch (Exception e) {
			logger.info("Seems that this indicator does not support the IndicatorLinesInterface");
			logger.info(e.toString());
			return null;
		}
		double height = this.getHeight()  - m_bMargin - m_tMargin;
		LogDataSeries logDS = m_cD.getLogDataSeries();
		double lx1;
		double ly1 = 0.0;
		double lx2;
		double ly2 = 0.0;
		if (indicatorAxisInfo.getScale().equals("linear")) {
			double absoluteValue = indicatorAxisInfo.getMaxValue() - indicatorAxisInfo.getMinValue();
			ly1 = height  +  m_tMargin - height * ( pL.getY1() -indicatorAxisInfo.getMinValue())/absoluteValue; //height /absoluteValue * pL.getY1();
			ly2 = height  +  m_tMargin - height * ( pL.getY2() -indicatorAxisInfo.getMinValue())/absoluteValue; //height /absoluteValue * pL.getY2();
		} else if (indicatorAxisInfo.getScale().equals("log")){
			double scaleY = height/(m_graphicsParameters.getLogaritmicMaximumValue() - m_graphicsParameters.getLogaritmicMinimumValue());
			ly1 =  (height + m_tMargin) - scaleY * (Math.log(pL.getY1())/Constants.log10 - m_graphicsParameters.getLogaritmicMinimumValue());
			ly2 =  (height + m_tMargin) - scaleY * (Math.log(pL.getY2())/Constants.log10 - m_graphicsParameters.getLogaritmicMinimumValue());
		}
		// Sets the current periode
		logDS.setCurrentPeriode(logDS.size() - 1 + m_graphicsParameters.getRelativeStartOfPeriode(), logDS.size() - 1 + m_graphicsParameters.getRelativeEndOfPeriode());
		int x = logDS.findNumberInPeriode(pL.getX1()) ;
		lx1 = x * pixPrDay + lMargin;
		x = logDS.findNumberInPeriode(pL.getX2());
		lx2 = x * pixPrDay + lMargin;
		Line2D l2D = new Line2D.Double(lx1, ly1, lx2, ly2);
		//System.out.println("x1, x2, y2: " + pL.getX1().toString() + ", " + pL.getX2().toString() + ", " + pL.getY2());
		return l2D;
	}
	
	public void moveLine(int x, int y) {
		// Should be moved out of here at some time
		PersistentLine pL = m_drawnLines.getActive();
		if (pL.isSelected()) {
			int selected = pL.getSelected();
			Line2D line = this.convertPersistentLine(pL);
			double lx1 = line.getX1();
			double ly1 = line.getY1();
			double lx2 = line.getX2();
			double ly2 = line.getY2();
			//System.out.println("Moving x, y: " + x + ", " + y);
			if (selected == 0) {
				lx1 = lx1 + x;
				ly1 = ly1 + y;
				lx2 = lx2 + x;
				ly2 = ly2 + y;
			} else if (selected == 1) {
				lx1 = lx1 + x;
				ly1 = ly1 + y;
			} else if (selected == 2) {
				lx2 = lx2 + x;
				ly2 = ly2 + y;
			}
			pL = convertPoints(lx1, ly1, lx2, ly2);
			m_drawnLines.setActive(pL);
			m_drawnLines.saveLines();
			logger.debug("Saving lines for " + m_cD.getTickerID().toString());
			repaint();
		}
	}
/**
 * Draws a temporary line while the mouse is dragged
 * 
 * @param x - end x coordinate for temporary line
 * @param y - end y coordinate for temporary line
 */	
    public void drawTemporaryLine(int x, int y) {
		m_x2 = x;
		m_y2 = y;
		// Don't draw lines less than 5 pixels long
		if (Math.abs(m_x1-m_x2) > 5 || Math.abs(m_y1-m_y2) > 5) {
			setVisible(true);
			repaint();               
		}
		
    }
/**
 * Sets the start point for a new line
 * 
 * @param x - start x coordinate for new line
 * @param y - start y coordinate for new line
 */
	public void startIndicatorLine(int x, int y) {
			m_x1 = x;
			m_y1 = y;
	}
	
	/**
	 * Sets the endpoint for a new line and saves the line
	 * The new line is displayed on the screen.
	 * 
	 * @param x - end x coordinate for new line
	 * @param y - end x coordinate for new line
	 */
	public void endIndicatorLine(int x, int y) {
		m_x2 = x;
		m_y2 = y;
			// TODO: Problems with lines (x-axis) beyond latest date
			// Partly fixed the initial draw part, but need to take into account that only working
			// days are drawn. Also need to fix the part that (re)draws the persistent lines. 
//				System.out.println(" Drawn line: " + x1 + " " + y1 + ", " + x2 + " " + y2);
			// Doesn't draw lines if neither x & y dimension is wider than 5 pixels
		if (Math.abs(m_x1-m_x2) > 5 || Math.abs(m_y1-m_y2) > 5) { 
			m_drawnLines.addLine(this.convertPoints(m_x1,m_y1,m_x2,m_y2));
			m_drawnLines.saveLines();
			this.repaint(); 
		}
		// No valid new line
		m_x1 = -1;         
	}

	public void panelRequestFocus() {
		//boolean focus = 
		this.requestFocusInWindow();
	}

	
	/**
	 * Saves whatever needs to be saved to the database.
	 */
	public void save() {
		m_drawnLines.saveLines();
	}

	/**
	 * Deleted selected line
	 */
	public void deleteSelectedLine() {
		m_drawnLines.deleteActiveSelection();
	}

	/**
	 * Select next line if selectNewLine is true, selects next end of the 
	 * current line if false
	 * 
	 * @param selectNewLine
	 * @return true if a line was selected, false otherwise
	 */
	public boolean selectNextLine(boolean selectNewLine) {
		boolean result;
		if (selectNewLine) {
			result = m_drawnLines.selectNext(false);
		} else {
			result = m_drawnLines.selectNextEnd();	
		}
		repaint();
		return result;
	}

}
