/*
 *                                 jStock
 *                    (C) Copyright 2003 - 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 28-Oct-2004
 * 
 */
package com.exponto.jstock.indicators;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.text.DecimalFormat;

import org.apache.log4j.Logger;

import com.exponto.jstock.gui.panels.GraphicsParameters;
import com.exponto.util.Date;

/**
 * CandleStick draws a logarithmic candlestick chart.
 * 
 * Note that this class doesn't have any indicator parameters (for now) and
 * returns null when asked for some.
 * 
 * @author kaare
 *
 */
public class CandleStick extends Indicator implements IndicatorInterface, IndicatorLinesInterface {
	static Logger logger = Logger.getLogger(CandleStick.class);
	private GraphicsParameters m_graphicsParameters;
	private LogDataSeries m_logDS = null;
	private static double sm_volumeScale;
	private static int sm_minorLineLevel;
	private int height; // height of the area useable for the candlesticks
	private int width; // width of the area useable for the candlesticks
	private boolean m_changed;
	private double m_HighInPeriode = -0.1;
	private double m_LowInPeriode = -0.1;
	private IndicatorAxisInfo m_iAI = null;

	public CandleStick() {
		super();
		sm_indicatorName = Constants.candlestick;
		//sm_indicatorColor = Color.green;
	}

	
	public static void setVolumeScale(double volumeScale) {
		sm_volumeScale = volumeScale;
	}
	
	public static void setMinorLineLevel(int minorLineLevel) {
		sm_minorLineLevel = minorLineLevel;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getIndicatorName()
	 */
	public String getIndicatorName() {
		return sm_indicatorName;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getIndicatorParameters()
	 */
	public IndicatorParameters getIndicatorParameters() {
		return null;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries)
	 */
	public void initiateIndicator(LogDataSeries lDS) {
		if (!lDS.equals(m_logDS)) {
			m_changed = true;
		}
		m_logDS = lDS;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries, com.exponto.jstock.indicators.IndicatorParameters)
	 */
	public void initiateIndicator(LogDataSeries lDS, IndicatorParameters indParam) {
		initiateIndicator(lDS);
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#initiateIndicator(com.exponto.jstock.LogDataSeries, java.lang.String)
	 */
	public void initiateIndicator(LogDataSeries lDS, String indParam) {
		initiateIndicator(lDS);
	}

	public boolean isMainIndicator() {
		return true;
	}
	
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#drawWithCandleSticks()
	 */
	public boolean drawWithCandleSticks() {
		return true;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#hasChanged()
	 */
	public boolean hasChanged() {
		return m_changed;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#paintIndicator(java.awt.Graphics2D, com.exponto.jstock.panels.GraphicsParameters)
	 */
	public void paintIndicator(Graphics2D g, GraphicsParameters gP) {
		Graphics2D g2d = g;
	    m_graphicsParameters = gP;
	    Dimension currentDim = m_graphicsParameters.getDimension();
	    int tMargin = m_graphicsParameters.getTopMargin();
	    int bMargin = m_graphicsParameters.getBottomMargin();
	    int rMargin = m_graphicsParameters.getRightMargin();
	    int lMargin = m_graphicsParameters.getLeftMargin();
		width = (int) currentDim.getWidth() - lMargin - rMargin;
		height = (int) currentDim.getHeight() - tMargin - bMargin;
		int pixPrDay = m_graphicsParameters.getPixelPrDay();
		int numberOfSticksToDisplay = width / pixPrDay;
		m_logDS.setCurrentPeriode(numberOfSticksToDisplay);
		// Draw numbers on x axis
	    if (m_logDS.isWeekly()) {
			drawWeeklyNumbers(g2d,  lMargin, tMargin, pixPrDay);
	    } else {
			drawWeekNumbers(g2d, lMargin, tMargin, pixPrDay);
	    }
	    m_HighInPeriode = m_logDS.tradedHighInPeriode;
	    m_LowInPeriode = m_logDS.tradedLowInPeriode;
	    m_iAI = new IndicatorAxisInfo(m_logDS.getYMin(), m_logDS.getYMax(), Constants.log);
		// Resetting this may cause serious harm, if the currentPeriode in logDS
		// isn't what you expected to be
	    m_logDS.findYScaleLimits(height);
		double logMin = Math.log(m_logDS.getYMin())/Constants.log10;
		double logMax = Math.log(m_logDS.getYMax())/Constants.log10; 
		m_graphicsParameters.setLogaritmicMaximumValue(logMax);
		m_graphicsParameters.setLogaritmicMinimumValue(logMin);
		double scaleY = height/(logMax - logMin);
		//end of reset
	    drawAxis(g2d, lMargin, tMargin, scaleY);
	    drawCandleSticks(g2d, scaleY);
		// User drawn lines
		//drawLines(g2d);
	    m_changed = false;
	 }
		
	    private void drawCandleSticks(final Graphics2D g2d, double scaleY){
	        int y0 = 34;
	        double h, y, x, sx1, sy1, sy2 = 0.0;
	        double scaleVol = 0.0;
	        double volPix = height * sm_volumeScale;
	        scaleVol = volPix/(m_logDS.getMaxVol());
			int tMargin = m_graphicsParameters.getTopMargin();
			int lMargin = m_graphicsParameters.getLeftMargin();
			int pixPrDay = m_graphicsParameters.getPixelPrDay();
			double logMin = m_graphicsParameters.getLogaritmicMinimumValue();
			//System.out.println("maks vol er:" + logDS.maxVol + " height: " + height);
	        y0 = height + tMargin;
	        LogTradingData lTD = new LogTradingData();
	        Line2D line = new Line2D.Double ();
	        Rectangle2D rect = new Rectangle2D.Double ();
	        m_logDS.resetCurrentPeriode();
	        int i = 0;
	      	while (m_logDS.hasNextLTD()) {
				lTD = m_logDS.getNextLTD();
				if (lTD == null) {
					logger.error("lTD is null, should not happend");  
//	 			System.out.println("CandleStickPanel, drawCandleSticks: Error - lTD is null");	
				} else if (!lTD.isUsable()) {
					if (lTD.getStatus() == Constants.faulty || lTD.getStatus() == Constants.drop) {
						g2d.setColor(Color.black);
						x = i * pixPrDay + lMargin;
						g2d.drawString( String.valueOf(lTD.getStatus()) , (float) x, tMargin + 6);
					}
	      		} else {
	      			// This is the normal
					y = y0 - scaleY*(lTD.logX-logMin);
					h = scaleY*lTD.logDiff;
					x = i * pixPrDay + lMargin;
					rect.setRect(x, y, pixPrDay -1, h);
					sx1 = x + pixPrDay/2 - 1;
					sy1 = y0 - scaleY*(lTD.logHigh-logMin);
					sy2 = y0 - scaleY*(lTD.logLow-logMin);
//					draws volume  
					g2d.setColor(Color.red);
					line.setLine(sx1,y0, sx1, (y0 - (lTD.volume * scaleVol)));
					g2d.draw(line);
					// Draws line between hi and low            
					g2d.setColor(Color.black);            
					line.setLine(sx1, sy1, sx1, sy2);
					g2d.draw(line);
//					Draws rectangle
					if (h == 0.0) {
						line.setLine(x, y, x + pixPrDay -1, y);
						g2d.draw(line);
					} else if (lTD.open < lTD.close) g2d.setColor(Color.white);
					g2d.fill(rect);
	      		}
				i++;	
	      	}
	    }
	        
	    private void drawAxis(final Graphics2D g2d, int lMargin, int tMargin, double scaleY) {
	        //TODO: Steps is int, should perhaps be a double
	        int x, yStart, yStop = 0;
	        //double minorSupportLineLevel = 60;
			DecimalFormat df = new DecimalFormat();
			df.setMaximumFractionDigits(2);
			df.setDecimalSeparatorAlwaysShown(false);
			g2d.setColor(Color.black);
	        x = lMargin;
	        yStart = tMargin;
	        yStop = yStart + height;
	        //Draws X-axis
	        Line2D line = new Line2D.Double (x, yStop, x + width, yStop);
	        g2d.draw(line);
	        //Draws Y-axis
	        line = new Line2D.Double(x, yStop, x, yStart) ;
	        g2d.draw(line);
			g2d.setColor(Color.DARK_GRAY);
			double previousY = yStop;
	        // Draw logaritmic axis from min to max
	        for (double i =  m_logDS.getYMin(); i <= m_logDS.getYMax(); i = i + m_logDS.stepSize) {
	            double y = height + tMargin - scaleY*((Math.log(i)/Constants.log10)-m_graphicsParameters.getLogaritmicMinimumValue());
	            line.setLine(width + lMargin, y, lMargin-5, y);
	            g2d.draw(line);
	            g2d.drawString(df.format(i), 10, (int) y+5);
	            if ((previousY - y) > sm_minorLineLevel) {
	            	// Draws the minor support levels
					previousY = y;
	            	double tempI = i - m_logDS.stepSize/2;
	            	y = height + tMargin - scaleY*((Math.log(tempI)/Constants.log10)-m_graphicsParameters.getLogaritmicMinimumValue());
					line.setLine(width + lMargin, y, lMargin-5, y);
					g2d.setColor(Color.GRAY);
					g2d.draw(line);
					g2d.drawString(df.format(tempI), 10, (int) y+5);
					g2d.setColor(Color.black);
	            } else {
					previousY = y;
	            }
	            if (i < m_logDS.stepSize) {
	            	// This fixes the problem with odd values on y axis due to large difference 
	            	// between high and low. See also findYScaleLimits() in LogDataSeries
	            	// First value will be odd, next ok
	            	i = 0;
	            }
	        }
			g2d.setColor(Color.black);
	    }     
	    
	    private void drawWeekNumbers(final Graphics2D g2d, int lMargin, int tMargin, int pixPrDay) {
	    	//logger.debug("Drawing week numbers");
			Line2D line = new Line2D.Double ();
			m_logDS.resetCurrentPeriode();
			int index  = m_graphicsParameters.getRelativeStartOfPeriode();
//			if (index < 0) {
//				index = 0;
//			}
			int numberOfPix = lMargin + m_logDS.numberOfDaysInFirstWeek(index) * pixPrDay;
//	 The first week, no line is currently drawn for that
			int currentWeek = m_logDS.getNextWeekNumber();
			//logger.debug("Current week: " + currentWeek);
			int pixPrWeek = 5 * pixPrDay;
			int y0 = height + tMargin;
			g2d.setColor(Color.black);
			currentWeek = m_logDS.getNextWeekNumber();
			//logger.debug("Current week: " + currentWeek);
			while (currentWeek != 0) {
				// Draws line before "current week"
				line.setLine(numberOfPix, y0, numberOfPix, y0+5);
				g2d.draw(line);
				numberOfPix = numberOfPix + pixPrWeek;
				//g2d.drawString(new Integer (currentWeek).toString(), (numberOfPix - (4 * pixPrDay)), y0+20);
				g2d.drawString(String.valueOf(currentWeek) , (numberOfPix - (4 * pixPrDay)), y0+20);
				currentWeek = m_logDS.getNextWeekNumber();
			}
			//logger.debug("Drawing week numbers done");
	    }

		private void drawWeeklyNumbers(final Graphics2D g2d, int lMargin, int tMargin, int pixPrDay) {
//			TODO: Bug - Align axis correctly. Seems that java & system calender does not agree on week numbers from 2000 and earlier 2001 and up seems ok.
			Line2D line = new Line2D.Double ();
			m_logDS.resetCurrentPeriode();
			// Start drawing at 3.rd stick
			int numberOfPix = lMargin;
			int currentWeek;
			int y0 = height + tMargin;
			int i = 1;
			g2d.setColor(Color.black);
			currentWeek = m_logDS.getNextWeekNumber();
			//System.out.println("F�rste hele uke: "+ currentWeek);
			while (currentWeek != 0) {
				numberOfPix = numberOfPix + pixPrDay;
				// draw every 5. th weeknumber
				if (i == 5) {
					line.setLine(numberOfPix + 1, y0, numberOfPix + 1, y0+i);
					g2d.draw(line);
					g2d.drawString(String.valueOf(currentWeek), (numberOfPix - 4), y0+20);
					i = 0;
				}
				i ++;
				currentWeek = m_logDS.getNextWeekNumber();
			}
		}
		public void markSelectedStick(final Graphics2D g2d, int number, int lMargin, int tMargin){
			int pixPrDay = m_graphicsParameters.getPixelPrDay();
			double logMin = m_graphicsParameters.getLogaritmicMinimumValue(); 
			double logMax = m_graphicsParameters.getLogaritmicMaximumValue(); 
			double scaleY = height/(logMax - logMin);
			if (number >= 0) {
				Line2D line = new Line2D.Double ();
				int x =lMargin + pixPrDay * number -1;
				int startPeriode = m_logDS.size() - 1 + m_graphicsParameters.getRelativeStartOfPeriode();
				if (startPeriode < 0) startPeriode = 0;
				double sy0 =	height + tMargin;
				LogTradingData lTD = m_logDS.getResult(startPeriode + number);
				double sy1 = sy0 - scaleY*(lTD.logHigh-logMin);
				// orange may give better visibility for the candlestick itself
				g2d.setColor(Color.gray);
				double y;
				boolean draw = true;
				// Draw dashed line
				for (y = sy1; y < sy0; y = y + 4) {
					if (draw) {
						line.setLine(x, y, x, y+4);    
						g2d.draw(line);
						line.setLine(x + pixPrDay, y, x + pixPrDay, y + 4);
						g2d.draw(line);
	 				}
					draw = !draw;
				}
				// Use this for solid line
				line.setLine(x, sy1, x + pixPrDay, sy1);
				g2d.draw(line);
				g2d.setColor(Color.black); 
			}
		}

		
	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#getDailyPrintOut(int)
	 */
	public String getDailyPrintOut(int n) {
		String s ="";
		String value ="NA";
		int index = 0;
		if (n < 0) {
			index = m_logDS.size() -1;
		} else {
			//System.out.println("index: " + index);
			int displayWidth = m_graphicsParameters.getNetWidth()/ m_graphicsParameters.getPixelPrDay();
			if (m_logDS.size() >= displayWidth) {
				// Series is longer than display width
				index = m_logDS.size() -1 - (displayWidth);
			}
			index = n + index;
		}
		if (index < m_logDS.size() && index >= 0) {
			LogTradingData lTD = m_logDS.getResult(index);
			if (lTD != null) {
				value = lTD.toString();
			}
		}
		return s + value;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#printAll(int)
	 */
	public String[] printAll(int number) {
		LogTradingData lTD;
		String [] sA = new String[number + 1];
		int start = m_logDS.size() - number;
		int index = 1;
		sA[0] = "Trading data is";
		if (start < 0) start = 0;
		for (int i = start; i < m_logDS.size(); i++) {
			lTD = m_logDS.getResult(i);
			sA[index++] = lTD.toString();
		}
		return sA;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorInterface#supportIndicatorValuesInterface()
	 */
	public boolean supportIndicatorValuesInterface() {
		return true;
	}

	public String toString() {
		double high, low;
		String selected;
		Date first;
		Date last;
		LogTradingData lTD;
		int position = m_logDS.size() - 1 + m_graphicsParameters.getRelativeStartOfPeriode();
		if (position < 0) position = 0;
		lTD = m_logDS.getResult(position);
		first = lTD.getDate(); 
		position = m_logDS.size() - 1 + m_graphicsParameters.getRelativeEndOfPeriode();
		if (position < 0) {
			// No reason to continue if no part of the indicator is currently displayed
			return "N/A";
		}
		lTD = m_logDS.getResult(position);
		last = lTD.getDate();
		high = m_HighInPeriode;
		low = m_LowInPeriode;
		String str = (" PERIODE: " + first.toString() + " - " + last.toString() + " High: " + String.valueOf(high) + " Low: " + String.valueOf(low));
		return str;
	}

	/* (non-Javadoc)
	 * @see com.exponto.jstock.indicators.IndicatorLinesInterface#getIndicatorAxisInfo()
	 */
	public IndicatorAxisInfo getIndicatorAxisInfo() {
		return m_iAI;
	}
	
}
