/*
 *                                 jStock
 *                    (C) Copyright 2003, 2004, Kaare Ranum
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials
 *      provided with the distribution.
 *
 *   3. Neither the name of Ex Ponto nor the names of its of its
 *      contributors may be used to endorse or promote products
 *      derived from this software without specific prior written
 *      permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/*
 * Created on 20-Oct-2004
 * Moved some old file io code here
 */
package com.exponto.jstock.io;

import java.io.File;
import java.io.IOException;
import java.net.URL;

import com.exponto.jstock.exception.TickerNotFoundException;
import com.exponto.jstock.indicators.LogDataSeries;

import org.apache.log4j.Logger;


/**
 * @author kaare
 *
 */
public class LoadDataSet {
	static Logger logger = Logger.getLogger(LoadDataSet.class);
	

	public LoadDataSet() {
		
	}
	
    /**
     * Called by external class to load a LogDataSeries either from file or from
     * the internet.
     * 
     * Does some preprocessing before a datafile is read. Among them
     * download the file if "internet" is enabled. Also check the format of the file. 
     * Then calls an other method to do the reading. 
     * 
     * @param tId 
     * @param numberOfDays Limits the number of days read if the file contains more data then wanted
     * @return LogDataSeries 
     * @throws TickerNotFoundException
     */
    public LogDataSeries loadData(TickerID tId, int numberOfDays) throws TickerNotFoundException {
    	UserPreferences userPrefs = UserPreferences.getInstance();
    	try {
    		numberOfDays = Integer.parseInt(userPrefs.getProperty(UserPreferences.numberOfDaysPropertyName)) ;            
    	} catch (NumberFormatException e) {
    		logger.error("Got a NumberFormatException when trying to read Number of Days from file, properties file probably corrupt, will continue with default value.");
    		userPrefs.setProperty(UserPreferences.numberOfDaysPropertyName, String.valueOf(numberOfDays)); 
    	}
    	// Just checks if the enableURL property is true or not
    	if (userPrefs.getPropertyBooleanValue(UserPreferences.enableURLPropertyName).booleanValue()) {
    		//URLModifiers.getURLModifierHashMap(tId.getTickerName(), tId.getExchangeName(), numberOfDays);
    		String rawUrl = userPrefs.getProperty(UserPreferences.urlPropertyName);
    		URL url = com.exponto.jstock.util.Util.URLCreator(rawUrl, URLModifiers.getURLModifierHashMap(tId.getTickerName(), tId.getExchangeName(), numberOfDays));
    		System.out.println(url.toString());
    		String fname = userPrefs.getFilePath() + tId.getTickerName() + "." + tId.getExchangeName();
    		File dataFile = new File(fname);
    		// Check if file exists
    		if (!dataFile.isFile()) {
    			try {
    				dataFile.createNewFile();
    			} catch (IOException e) {
    				logger.error(" Failed to create " + fname + " due to " +e.toString());
    			}
    			logger.debug("Created new datafile: " + fname);				
    		} else {
    			logger.debug("Overwriting existing datafile: " + fname);
    		}
    		// Download file
     		try {
    			com.exponto.jstock.util.Util.downloadURL(url, dataFile);
    		} catch (Exception e) {
    			logger.error("Couldn't download datafile from "+ url.toString() +", tries to use previously download file instead");
    		}
    	}
    	int format;
    	String fileFormat = userPrefs.getProperty(UserPreferences.fileFormatPropertyName) ;
    	if (fileFormat.equalsIgnoreCase(Constants.netfonds)) {
    		format = 0;
    	} else if (fileFormat.equalsIgnoreCase(Constants.yahoo)) {
    		format = 1;
    	} else if (fileFormat.equalsIgnoreCase(Constants.other)) {
    		format = 2;
    	} else {
    		format = 99;
    	}
        return this.loadLogDataSeries(numberOfDays, tId, format);

    }

    /**
	 * Reads dataseries from a file
 	 *
     * @param numberToGet
     * @param tId
     * @param format
     * @return
     * @throws TickerNotFoundException
     */
	private LogDataSeries loadLogDataSeries(int numberToGet, TickerID tId, int format) throws TickerNotFoundException {
		LogDataSeries lDS = readFile(tId, numberToGet, format);
		// The first shall be last etc.
		lDS.rotate();
		if (!Util.validate(lDS)) {
			logger.error("File " + tId.toString() + " didn't validate correctly");
		}
		return lDS;
	}

    /**
     * Reads a file and converts it to a LogDataSeries. 
     * 
     * @param str is the complete filename of the file to be read
     * @param numberToGet determines how many days that is read from the file
     * @return  a LogDataSeries generated from data */    
    private LogDataSeries readFile(TickerID tID, int numberToGet, int format) throws TickerNotFoundException {
        LogDataSeries lDS;
        UserPreferences uP = UserPreferences.getInstance();
        String fname = uP.getFilePath() + tID.getTickerName() + "." + tID.getExchangeName();
        File testFile = new File(fname);
        if (!testFile.isFile()) {
        	// Tries with extension as upper case if file is not found at first try
        	fname = uP.getFilePath() + tID.getTickerName() + "." + tID.getExchangeName().toUpperCase();
        }
        // Not future proof as everything else than the "other" format is treated as Netfonds.
        // Works for Netfonds and Yahoo format.
        if (format != 2) {
        	ReadNetfondsFile rNF = new ReadNetfondsFile();
        	lDS = rNF.readSeries(fname, numberToGet, format);
        } else {
        	ReadGenericDataFile rGDF = new ReadGenericDataFile();
        	lDS = rGDF.readSeries(fname, numberToGet);
        }
		logger.debug("File loaded" + fname);	
        return lDS;
    }
}
